package com.bcxin.backend.domain.auth.service.impl;

import com.bcxin.backend.core.exceptions.SaasBadException;
import com.bcxin.backend.domain.auth.dtos.AuthLog;
import com.bcxin.backend.domain.auth.dtos.AuthRequestDto;
import com.bcxin.backend.domain.auth.factory.AuthFactory;
import com.bcxin.backend.domain.models.Result;

import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;


import java.util.*;


@Service
@Slf4j
public abstract class AuthenticationServiceAbstractImpl {

    private static Logger logger = LoggerFactory.getLogger(AuthenticationServiceAbstractImpl.class);

    private final AuthFactory authFactory;

    @Autowired
    @Qualifier("primaryJdbcTemplate")
    protected JdbcTemplate jdbcTemplate;

    public AuthenticationServiceAbstractImpl(AuthFactory authFactory) {
        this.authFactory = authFactory;
    }

    public abstract String findUnAuthSql();

    public Boolean auth() {
        logger.info("-----------------------------------------------实名认证程序开始执行--------------------------------------------------------------");
        logger.info("-----------------------------------------------组装人员信息开始执行--------------------------------------------------------------");
        //查询出来待认证人员
        try {
            List<AuthRequestDto> authRequestDtos = this.getPendingBgScreenUsers();
            logger.info("查询数据库，待认证人员条数：" + authRequestDtos.size());
            System.out.println("查询数据库，待认证人员条数：" + authRequestDtos.size());
            if (!CollectionUtils.isEmpty(authRequestDtos)) {
                List<AuthRequestDto> limitAuthUsers = Collections.synchronizedList(new ArrayList<>());
                for (AuthRequestDto authRequestDto :authRequestDtos){
                    limitAuthUsers.add(authRequestDto);
                    if (limitAuthUsers.size()==5){
                        try {
                            check(limitAuthUsers);
                        }catch (Exception e) {
                            logger.error("分页实名认证check方法运行异常:" + e.getMessage());
                        }
                        limitAuthUsers = Collections.synchronizedList(new ArrayList<>());
                    }
                }
                if (limitAuthUsers.size()>0){
                    try {
                        check(limitAuthUsers);
                    }catch (Exception e) {
                        logger.error("分页实名认证check方法运行异常:" + e.getMessage());
                    }
                }

                logger.info("-----------------------------------------------实名认证程序结束--------------------------------------------------------------");
                return true;
            } else {
                System.err.println("被筛人员列表为空");
                logger.info("被筛人员列表为空");
                logger.info("-----------------------------------------------实名认证程序结束--------------------------------------------------------------");

                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e.getMessage());
            logger.info("-----------------------------------------------实名认证程序结束--------------------------------------------------------------");
            return false;
        }
    }

    private List<AuthRequestDto> getPendingBgScreenUsers() {
        System.out.println(findUnAuthSql());
        return jdbcTemplate.query(findUnAuthSql(), BeanPropertyRowMapper.newInstance(AuthRequestDto.class));
    }


    private Result<List<AuthLog>> doExecute(Collection<AuthRequestDto> authRequestDtos) {
        return authFactory.getAuthByType().auth(authRequestDtos);
    }

    protected void check(List<AuthRequestDto> authRequestDtos) {
        logger.info("-----------------------------------------------对比程序开始执行--------------------------------------------------------------");
        try {
            List<AuthLog> list = new ArrayList<>();
            //System.out.println("待认证人员"+authRequestDtos);
            Result<List<AuthLog>> result = this.doExecute(authRequestDtos);
            //System.out.println("实名认证结果"+result.toString());
            if (Result.SUCCESS.equals(result.getRetType())) {
                if (result.getData() == null) {
                    throw new SaasBadException(String.format("AuthenticationServiceAbstractImpl:Request interface message returns exception,ErrMsg:%s.", result.getMsg()));
                }
                //获取查询出来的数据
                list = result.getData();
                //存问题人员身份证号码，方便通过身份证号码查询添加问题人员所需要的企业信息
                /*List<String> problemidnums = new ArrayList<>();
                //存问题人员的问题类型。
                Map<String, String> problemMap = new HashMap<>();
                for (AuthLog authlog : list
                ) {
                    if (!StringUtils.isEmpty(authlog.getAuthResult())) {
                        if (authlog.getAuthStatus().equals("2")) {
                            problemidnums.add("'" + authlog.getIdNumber() + "'");
                            problemMap.put(authlog.getIdNumber(), authlog.getAuthResult());
                        }
                    }
                    System.err.println(authlog.getIdNumber()+"的问题类型："+authlog.getAuthResult());
                }*/

                //保存认证结果
                if (list.size() > 0) {
                    for (AuthLog authlog : list) {
                        this.updateAuthResult(authlog);
                    }
                }
                //有问题，新增问题记录
               /* if (problemidnums.size() > 0) {
                    this.addProblemPeople(problemidnums, problemMap);
                }*/
            } else {
                throw new SaasBadException(String.format("AuthenticationServiceAbstractImpl:Request interface failed,ErrMsg:%s.", result.getMsg()));

            }
        } catch (Exception e) {
            logger.error("实名认证出现异常:" + e.getMessage());
            e.printStackTrace();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateAuthResult(AuthLog authlog){
        String sql2 = "UPDATE bg_screening_users SET `authenticated_status`=?,`authenticated_time`=NOW()," +
                " `last_modified_time`=NOW(),`authenticated_result`= ? WHERE `idnum` = ?";
        jdbcTemplate.update(sql2, authlog.getAuthStatus(), authlog.getAuthResult(), authlog.getIdNumber());
        /*String sql3 = "UPDATE tlk_securityman SET `ITEM_shiming`=?,`ITEM_beizhu`=?,`ITEM_LastAuthTime`=NOW()," +
                "`LASTMODIFIED`=NOW() WHERE `ITEM_documentID`=? AND `ITEM_isInMyCompany`='1';";
        String oldAuthStatus="";
        if (authlog.getAuthStatus().equals("2")){
            oldAuthStatus="认证失败";
        }else if (authlog.getAuthStatus().equals("3")){
            oldAuthStatus="已认证";
        }else {
            oldAuthStatus="认证中";
        }
        jdbcTemplate.update(sql3,oldAuthStatus, authlog.getAuthResult(), authlog.getIdNumber());*/
    }

    //添加问题人员记录
    private void addProblemPeople(List<String> problemidnums, Map<String, String> problemMap) {
        //筛选要生成问题人员记录的数据
        String problemSql = "SELECT " +
                "bs.idnum as 'idNumber' ,tm.domainid as 'domain_id',tm.item_domain_id as 'organization_id',tm.item_companyName as 'companyName',tm.ITEM_entryDate as 'entryTime', " +
                "tm.ITEM_phone as 'phone',bs.`name` as 'securityName' ,bs.one_inch_color_white_photo as 'imgPath', " +
                "(SELECT ifnull(ts.ITEM_registerPoliceAddress,'') FROM tlk_setsupervise ts WHERE ts.item_domain_id = tm.ITEM_DOMAIN_ID limit 1)  as registerPoliceAddress," +
                "(SELECT ifnull(ts.ITEM_registerPoliceAddressID,'') FROM tlk_setsupervise ts WHERE ts.item_domain_id = tm.ITEM_DOMAIN_ID limit 1)  as registerPoliceAddressID " +
                "FROM " +
                "tlk_securityman tm " +
                "INNER JOIN bg_screening_users bs ON LEFT (tm.ID, LENGTH(bs.user_id)) = bs.user_id " +
                "WHERE " +
                "tm.ITEM_documentID in (" + problemidnums.toString().replace("[", "")
                .replace("]", "") + ") " +
                "AND tm.ITEM_isInMyCompany = 1;";
        System.err.println("查询数据库，需要添加问题人员sql：" + problemSql);
        List<AuthLog> problemList = jdbcTemplate.query(problemSql, BeanPropertyRowMapper.newInstance(AuthLog.class));
        System.err.println("查询数据库，需要添加问题人员条数：" + problemList.size());
        if (problemList.size()>0) {
            StringBuilder sb = new StringBuilder();
            List<String> argList = Collections.synchronizedList(new ArrayList<>());
            sb.append("INSERT INTO bg_screening_user_results (ID,TYPE,CREATED,lastmodified,firstcompareTime,DOMAINID," +
                    "QUESTIONTYPE,SECURITYNAME,DOCUMENTID,PHONE,COMPANYNAME," +
                    "DOMAIN_ID,REGISTERPOLICEADDRESS," +
                    "REGISTERPOLICEADDRESSID,POLICESTATUS,WORKSTATUS,COMPANYSTATUS,Entry_Time,COMPARETIME) values");
            for (AuthLog authlog : problemList
            ) {
                String eTime = "'" + authlog.getEntryTime() + "'";
                if (StringUtils.isEmpty(authlog.getEntryTime())) {
                    eTime = null;
                }
                String rpa = "'" + authlog.getRegisterPoliceAddress() + "'";
                if (StringUtils.isEmpty(authlog.getRegisterPoliceAddress())) {
                    rpa = null;
                }
                String rpai = "'" + authlog.getRegisterPoliceAddressID() + "'";
                if (StringUtils.isEmpty(authlog.getRegisterPoliceAddressID())) {
                    rpai = null;
                }
                String phone = "'" + authlog.getPhone() + "'";
                if (StringUtils.isEmpty(authlog.getPhone())) {
                    phone = null;
                }
                String cn = "'" + authlog.getCompanyName() + "'";
                if (StringUtils.isEmpty(authlog.getCompanyName())) {
                    cn = null;
                }
                String un = "'" + authlog.getSecurityName() + "'";
                if (StringUtils.isEmpty(authlog.getSecurityName())) {
                    un = null;
                }
                String cid = "'" + authlog.getOrganization_id() + "'";
                if (StringUtils.isEmpty(authlog.getOrganization_id())) {
                    cid = null;
                }
                String domainid = "'" + authlog.getDomain_id() + "'";
                if (StringUtils.isEmpty(authlog.getDomain_id())) {
                    domainid = null;
                }
                String sql = "( UUID_SHORT() ,'4', NOW(),NOW(), NOW(),"+domainid+" , '"
                        + problemMap.get(authlog.getIdNumber()) + "'," + un + ",'" +
                        authlog.getIdNumber() + "', " + phone + "," + cn +
                        "," + cid + ", " + rpa + "," + rpai + ",0,1,0," + eTime + ",NOW() )";
                argList.add(sql);
            }
            sb.append(argList.toString().replace("[", "").replace("]", ";"));
            System.err.println("添加问题人员sql：" + sb.toString());
            jdbcTemplate.update(sb.toString());
        }
    }
}
