package com.bcxin.backend.domain.services.impls;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.bcxin.backend.domain.services.ConvertService;
import com.bcxin.backend.domain.syncs.dtos.SharedConvertDto;
import com.bcxin.backend.domain.utils.PDFUtils;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.text.SimpleDateFormat;
import java.util.List;
@Slf4j
@Service
public class ConvertServiceImpl implements ConvertService {

    @Value("${myapps.storage.root}")
    String rootPath;

    /**
     * <b> html转pdf </b>
     * 使用openhtmltopdf技术
     * @param tempUrl http:xxxx/xxx.html
     * @param formData {..json..}
     * @author ZXF
     * @create 2022/10/26 0026 16:36
     * @version
     * @注意事项 </b>
     */
    public String execConvertHtmltopdfForTemp(SharedConvertDto convertDto) throws IOException {
        Long a = System.currentTimeMillis();
        if(StringUtils.isEmpty(convertDto.getTempUrl())|| ObjectUtil.isEmpty(convertDto.getFormData())){
            return null;
        }
        JSONObject data = JSONObject.fromObject(convertDto.getFormData());
        String pdfPath =  ObjectUtil.isEmpty(data.get("unSignaturePDF"))?"":data.get("unSignaturePDF")+"";
        if(StringUtils.isEmpty(pdfPath)){
            return null;
        }
        String pdfFilePath = rootPath + pdfPath;
        File pdfFile = new File(pdfFilePath);
        if(!pdfFile.getParentFile().exists()){
            pdfFile.getParentFile().mkdirs();
        }
        PDFUtils.generatePdfByOpenhtmltopdf(convertDto.getTempUrl(),pdfFilePath,convertDto.getFormData());
        pdfFile = new File(pdfFilePath);
        if(pdfFile.length()==0){
            pdfFile.delete();
            return null;
        }
        String newPath = pdfFilePath.replace(".temp",".pdf");
        File newFile = new File(newPath);
        //重命名这个路径的文件
        boolean flag = pdfFile.renameTo(newFile);
        if(!flag){
            pdfFile.delete();
            return null;
        }
        log.info("====> 电子签章定时任务开始.createPDF.execConvertHtmltopdfForTemp：Time："+ (System.currentTimeMillis()-a));
        return newPath;
    }

    public static void main(String[] args) throws IOException {
        SharedConvertDto convertDto = new SharedConvertDto();
        convertDto.setTempUrl("https://v5qy.de.baibaodun.com.cn/static/signon/elesign/qualicerttemp_zgz.html");
//        convertDto.setFormData("{\"address\":\"山东省阳谷县高庙王镇邵楼村774号\",\"certificateno\":\"京2023123227\",\"day\":\"02\",\"fzDate\":\"2023-01-06\",\"headphoto\":\"https://02obs-file-system-obpm-uploads.obs.cn-north-1.myhuaweicloud.com/2022/10-12/c9a018e2-082a-4afa-b2d4-b761763dff08/1665571917323.jpg\",\"idcardno\":\"371521200403026118\",\"isSignature\":\"1\",\"month\":\"03\",\"name\":\"邵贵靖\",\"unSignaturePDF\":\"/uploads/pdf/dianziqianzhang/temp/queueId_2604.pdf\",\"year\":\"2004\",\"zDay\":\"06\",\"zMonth\":\"01\",\"zYear\":\"2023\"}");
//        convertDto.setFormData("{\"address\":\"山西省壶关县集店镇东长井村3队260号\",\"certificateno\":\"京2023254630\",\"day\":\"24\",\"fzDate\":\"2023-07-17\",\"headphoto\":\"https://02obs-file-system-obpm-uploads.obs.cn-north-1.myhuaweicloud.com/2023/07-03/61dccdc3-f984-4255-8b7d-8bb89425c694/1688356955458.jpg\",\"idcardno\":\"140427200211241234\",\"isSignature\":\"1\",\"month\":\"11\",\"name\":\"曹速达\",\"unSignaturePDF\":\"/uploads/pdf/dianziqianzhang/temp/queueId_2604.pdf\",\"year\":\"2002\",\"zDay\":\"17\",\"zMonth\":\"07\",\"zYear\":\"2023\"}");
        String con = "{\"address\":\"山西省河津市樊村镇干涧村\",\"certificateno\":\"京2023254021\",\"day\":\"15\",\"fzDate\":\"2023-07-18\",\"headphoto\":\"https://02obs-file-system-obpm-uploads.obs.cn-north-1.myhuaweicloud.com/2023/07-03/61dccdc3-f984-4255-8b7d-8bb89425c694/1688356955458.jpg\",\"idcardno\":\"140882200411150018\",\"isSignature\":\"1\",\"month\":\"11\",\"name\":\"史梓彬\",\"unSignaturePDF\":\"/uploads/pdf/dianziqianzhang/temp/queueId_205459.temp\",\"year\":\"2004\",\"zDay\":\"18\",\"zMonth\":\"07\",\"zYear\":\"2023\"}";
        convertDto.setFormData(JSON.parse(con));
        PDFUtils.generatePdfByOpenhtmltopdf(convertDto.getTempUrl(),"D:\\data\\upload\\a8.pdf", convertDto.getFormData());
    }

    /**
     * <b> 遍历临时目录下的pdf，取第一个剪切到in目录 </b>
     * @param convertDto 里面就传临时目录路径
     * @return null,剩余文件个数,文件名
     * @author ZXF
     * @create 2023/01/07 0007 11:14
     * @version
     * @注意事项 </b>
     */
    public String existsPdfCopy(SharedConvertDto convertDto) {
        int count = 80;
        if (convertDto.getFormData()!=null){
            count = Integer.parseInt(String.valueOf(convertDto.getFormData()));
        }
        if(StringUtils.isEmpty(convertDto.getTempUrl())){
            return null;
        }
        //临时目录
        String tempFilePath = rootPath + convertDto.getTempUrl();
        String inFilePath = rootPath + convertDto.getTempUrl().replace("/temp","/in");
        File tempFile = new File(tempFilePath);
        if (!tempFile.exists()) {
            return null;
        }
        //in目录
        File inFile = new File(inFilePath);
        if (!inFile.exists()) {
            return null;
        }
        File[] fi = inFile.listFiles();
        if (fi.length>20){
            log.info("=======> 签章业务：剪切文件到指定目录异常，剩余文件超20个，打回");
            return null;
        }
        File[] fa = tempFile.listFiles();
        int pCount = 0;
        //新建一个String数组用于存放所有文件名
        List<String> list = Lists.newArrayList();
        for(int i = 0; i < fa.length; i++){
            if(fa[i].getName().contains(".pdf")&&pCount<count){
                list.add(fa[i].getName());
                pCount++;
            }
        }
        if(list.size() == 0){
            return null;
        }
        //新建一个String数组用于存放所有文件名
//        String[] fileNames = list.toArray(new String[list.size()]);
        //收集剪切成功的
        List<String> usrlist = Lists.newArrayList();
        //将临时目录下的pdf文件剪切到in目录下(覆盖已有的目标路径)
        for(String fileName : list){
            try {
                File temp = new File(tempFilePath + "/" + fileName);
                if (!temp.exists()) {
                    continue;
                }
                Files.move(Paths.get(tempFilePath + "/" + fileName), Paths.get(inFilePath + "/" + fileName), StandardCopyOption.REPLACE_EXISTING);
                usrlist.add(fileName);
            } catch (IOException e) {
                log.info("=======> 签章业务：剪切文件到指定目录异常，文件名："+ fileName +"，ERROR："+e.getMessage());
            }
        }
        log.info("=======> 签章业务：剪切文件到指定目录异常，已剪切："+ usrlist.size() +"个");
        /*for (int i = 0; i < count; i++) {
            try {
                Files.move(Paths.get(tempFilePath + "/" + fileNames[i]), Paths.get(inFilePath + "/" + fileNames[i]), StandardCopyOption.REPLACE_EXISTING);
            } catch (IOException e) {
                System.err.println("=======> 签章业务：剪切文件到指定目录异常，文件名："+ fileNames[i] +"，ERROR："+e.getMessage());
            }
        }*/
        StringBuilder result = new StringBuilder();
        for (String fileName : usrlist) {
            result.append(fileName);
            result.append(";");
        }
        return "0,"+result;
    }

    public void delInPDF(SharedConvertDto convertDto) {
        String inFilePath = rootPath + convertDto.getTempUrl().replace("/temp","/in");
        String outFilePath = rootPath + convertDto.getTempUrl().replace("/temp","/out");
        //in目录
        File inFile = new File(inFilePath);
        if (!inFile.exists()) {
            return;
        }
        //out目录
        File outFile = new File(outFilePath);
        if (!outFile.exists()) {
            return;
        }
        File[] ifi = inFile.listFiles();
        File[] ofi = outFile.listFiles();
        if (ifi.length!=0&&ofi.length!=0){
            String outFName = "";
            for (int i = 0; i < ofi.length; i++) {
                File fs = ofi[i];
                outFName = outFName + fs.getName();
            }
            if(StringUtils.isNotEmpty(outFName)){
                for (int i = 0; i < ifi.length; i++) {
                    File fs = ifi[i];
                    if(outFName.contains(fs.getName())){
                        if (fs.exists()) {
                            fs.delete();
                        }
                    }
                }
            }
        }
    }

    public void delSetFtpFile(String tempUrl,String fileNames) {
        String inFilePath = rootPath + tempUrl.replace("/temp","/in");
        String outFilePath = rootPath + tempUrl.replace("/temp","/out");
        //in目录
        File inFile = new File(inFilePath);
        if (!inFile.exists()) {
            return;
        }
        //out目录
        File outFile = new File(outFilePath);
        if (!outFile.exists()) {
            return;
        }
        File[] ifi = inFile.listFiles();
        if (ifi.length!=0){
            for (int i = 0; i < ifi.length; i++) {
                File fs = ifi[i];
                if(fileNames.contains(fs.getName())){
                    if (fs.exists()) {
                        fs.delete();
                    }
                }
            }
        }
        File[] ofi = outFile.listFiles();
        if (ofi.length!=0){
            for (int i = 0; i < ofi.length; i++) {
                File fs = ofi[i];
                if(fileNames.contains(fs.getName())){
                    if (fs.exists()) {
                        fs.delete();
                    }
                }
            }
        }
    }

    /**
     * <b> 单独检索in目录0kb文件删除 </b>
     * @author ZXF
     * @create 2023/07/19 0019 10:46
     * @version
     * @注意事项 </b>
     */
    public void delInPDF(String tempUrl) {
        String inFilePath = rootPath + tempUrl.replace("/temp","/in");
        //in目录
        File inFile = new File(inFilePath);
        if (!inFile.exists()) {
            return;
        }
        File[] ifi = inFile.listFiles();
        for (int i = 0; i < ifi.length; i++) {
            File fs = ifi[i];
            if(fs.length()<5){
                fs.delete();
            }
        }
    }

    public String existsScanSignature(SharedConvertDto convertDto) {
        if(StringUtils.isEmpty(convertDto.getTempUrl())){
            return null;
        }
        return PDFUtils.uploadFtpFile(convertDto.getTempUrl(),rootPath);
    }
}
