package com.bcxin.backend.domain.signature.service.impls;

import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.backend.core.utils.ExceptionUtils;
import com.bcxin.backend.domain.models.SignatureQueuesDTO;
import com.bcxin.backend.domain.repositories.SignatureQueuesRepository;
import com.bcxin.backend.domain.services.impls.ConvertServiceImpl;
import com.bcxin.backend.domain.signature.service.SignatureToPDFService;
import com.bcxin.backend.domain.syncs.dtos.SharedConvertDto;
import io.jsonwebtoken.lang.Collections;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.*;

@Slf4j
@Service
public class SignatureToPDFServiceImpl implements SignatureToPDFService {

//    public static Collection<SignatureQueuesDTO> sqList = new ArrayList<>();
    private final String cgiApi = "https://192.168.60.61:8890/XMPSignQuicklyExCGI.cgi";
    private final SignatureQueuesRepository signatureQueuesRepository;
    @Value("${myapps.domain.url}")
    String urlHead;
//    @Value("${myapps.signature.api}")
//    String api;
//    @Value("${myapps.signature.apiScan}")
//    String apiScan;
//    @Value("${myapps.signature.apiCopy}")
//    String apiCopy;
    @Value("${myapps.signature.unSignaturePDF}")
    String unSignatureSignatureToPDFServiceImplPDF;
    @Value("${myapps.signature.signaturePDF}")
    String signaturePDF;
    @Value("${myapps.signature.tempPDF}")
    String tempPDF;
    @Autowired
    private ConvertServiceImpl convertService;

    @Autowired
    @Qualifier("primaryJdbcTemplate")
    protected JdbcTemplate jdbcTemplate;

    public SignatureToPDFServiceImpl(SignatureQueuesRepository signatureQueuesRepository) {
        this.signatureQueuesRepository = signatureQueuesRepository;
    }

    @Override
    public void cerPDF() {
        //后执行要创建签章中业务的逻辑
        createPDF();
    }

    /**
     * <b> 激活电子印章签验接口服务器完成签章 </b>
     * @author ZXF
     * @create 2022/11/21 0021 10:44
     * @version
     * @注意事项 </b>
     */
    private boolean activateXMPSignQuicklyExCGI() {
        StringBuilder sb = new StringBuilder();
        try {
            JSONObject json = new JSONObject();
            json.put("service", "wss://127.0.0.1:8800");
            json.put("sn", "2171101001009256");
            json.put("pin", "88888888");
            json.put("in", "/data/share/in");
            json.put("out", "/data/share/out");
            JSONArray arr = new JSONArray();
            JSONObject pos = new JSONObject();
            pos.put("centerX", 405);
            pos.put("centerY", 312);
            pos.put("page", 1);
            arr.add(pos);
            json.put("signOnPos", arr);
            String ret = HttpUtil.post(cgiApi, json.toJSONString());
            log.info("====> 电子签章定时任务开始.activateXMPSignQuicklyExCGI.ret：" + ret);
            if (StringUtils.isEmpty(ret)) {
                return false;
            }
            JSONObject result = JSON.parseObject(ret.replace("#", ""));
            if ("1".equals(String.valueOf(result.get("code")))) {
                return true;
            }
            return false;
        } catch (Exception e){
            log.info("====> 电子签章定时任务开始.activateXMPSignQuicklyExCGI message:"+e.getMessage());
        }
        return false;
    }

    /**
     * <b> 批量生成pdf文件 </b>
     * @author ZXF
     * @create 2022/10/31 0031 13:33
     * @version
     * @注意事项 </b>
     */
    private void createPDF() {
        //1.查询签章队列表需要签章的数据
        Page<SignatureQueuesDTO> pageQueues = signatureQueuesRepository.pageQueues("0", PageRequest.of(0, 300));
        List<SignatureQueuesDTO> queues = pageQueues.getContent();
        //Collection<SignatureQueuesDTO> queues = pageQueues.getContent();
        if (Collections.isEmpty(queues)) {
            return;
        }
        for (SignatureQueuesDTO queuesDTO:queues) {
            accept(queuesDTO);
        }
//        Collection<Future> futures = new ArrayList<>();
//        for (int i = 1; i <= queues.size(); i++) {
//            int finalI = i;
//            CompletableFuture f =
//                    CompletableFuture.runAsync(() -> {
//                        SignatureQueuesDTO queuesDTO = queues.get(finalI);
//                        this.accept(queuesDTO);
//                    });
//            futures.add(f);
//        }
//        System.err.println("=========>>已分配内存: "+Runtime.getRuntime().totalMemory()/1024/1024+"m  已分配内存中的剩余空间: "+ Runtime.getRuntime().freeMemory()/1024/1024+"m  最大可用内存: "+
//                (Runtime.getRuntime().maxMemory()-Runtime.getRuntime().totalMemory()+Runtime.getRuntime().freeMemory())/1024/1024+"m<<===========");

        //2.先判断一共查询了多少条，然后再判断要循环多少次
        /*int count = ((queues.size()) / 10);
        System.err.println("=============> 电子签章定时任务第四行count取值执行完毕,count的值为： <============="+count);
        if(queues.size()%10!=0) {
            count += 1;
        }
        System.err.println("=============> 电子签章定时任务第五行count余数取值执行完毕,count的值为： <============="+count);
        List<String> arrayList;
        System.err.println("=============> 电子签章定时任务第六行定义arrayList执行完毕 <=============");
        //3.将查询到的数据通过每十条为一个线程的方式开启循环调生成pdf接口
        Collection<Future> futures = new ArrayList<>();
        Long time = System.currentTimeMillis();
        for (int i = 1; i <= count; i++) {
            arrayList = new ArrayList<>();
            System.err.println("=============> 电子签章定时任务第七行new ArrayList执行完毕,arrayList的值为： <============="+arrayList);
            for (int j = (i-1)*10; j < i*10; j++) {
                if(j==queues.size()){
                    break;
                }else{
                    arrayList.add(queues.get(j));
                }
            }
            System.err.println("=============> 电子签章定时任务第八行arrayList添加元素执行完毕,arrayList的值为： <============="+arrayList);
            List<String> runList = arrayList;
            int p = i;
            System.err.println("=============> 电子签章定时任务第九行新建一个runList并且引用指向arrayList执行完毕,runList的值为： <============="+runList);
            CompletableFuture f =
                    CompletableFuture.runAsync(() -> {
                        for (String s : runList) {
                            try {
                                accept2(s);
                            } catch (InterruptedException e) {
                                e.printStackTrace();
                            }
                        }
                        System.err.println("=================>完成第"+p+"组循环花费时间"+(System.currentTimeMillis()-time)/1000+"<================");
                    });
            futures.add(f);
        }*/
    }

    /**
     * <b> 批量确认签章pdf并更新证书信息 </b>
     * @author ZXF
     * @create 2022/10/31 0031 13:33
     * @version
     * @注意事项 </b>
     */
    /*private void signaturePDF() {
        //1.查询签章队列表等待签章的数据
//        Page<SignatureQueuesDTO> pageQueues = signatureQueuesRepository.pageQueues("3",PageRequest.of(0, 200));
//        Collection<SignatureQueuesDTO> queues = pageQueues.getContent();
        System.err.println("====> 电子签章定时任务开始.signaturePDF.queues.size："+sqList.size());
        if(Collections.isEmpty(sqList)){
            return;
        }
        //2.循环调确认已签章pdf文件接口（不管有没有成功，这个接口中的业务都会将存在的记录删除）
        sqList.parallelStream().forEach(this::scan);
        //不管有没有签章成功都清空队列
        sqList = new ArrayList<>();
    }*/

    public static void main(String[] args) {

        /*List<String> queues = new ArrayList<>();
        for (int i = 0; i < 101; i++) {
            queues.add(i+"");
        }
        //2.先判断一共查询了多少条，然后再判断要循环多少次
        int count = ((queues.size()) / 10);
        if(queues.size()%10!=0) {
            count += 1;
        }
        List<String> arrayList;
        //3.将查询到的数据通过每十条为一个线程的方式开启循环调生成pdf接口
        for (int i = 1; i <= count; i++) {
            arrayList = new ArrayList<>();
            for (int j = (i-1)*10; j < i*10; j++) {
                if(j==queues.size()){
                    break;
                }else{
                    arrayList.add(queues.get(j));
                }
            }
            System.out.println(arrayList);
        }
        System.out.println("count为："+count);*/
        //4.循环调生成pdf接口
        //queues.forEach(this::accept);
        /*JSONObject json = new JSONObject();
        json.put("service","wss://127.0.0.1:8800");
        json.put("sn","2171101001009256");
        json.put("pin","88888888");
        json.put("in","/data/share/in");
        json.put("out","/data/share/out");
        JSONArray arr = new JSONArray();
        JSONObject pos = new JSONObject();
        pos.put("centerX","369511");
        pos.put("centerY","488629");
        pos.put("page","1");
        arr.add(pos);
        json.put("signOnPos",arr);
        System.out.println(json.toJSONString());*/
        /*JSONObject j = new JSONObject();
        SignatureData data = new SignatureData();
        data.setName("张小国");
        data.setIdcardno("132302197104241418");
        data.setAddress("北京市市辖区朝阳区来广营臻园小区");
        data.setHeadphoto("https://02obs-file-system-obpm-uploads.obs.cn-north-1.myhuaweicloud.com/2022/10-12/c9a018e2-082a-4afa-b2d4-b761763dff08/1665571917323.jpg");
        data.setYear("1971");
        data.setMonth("04");
        data.setDay("24");
        data.setCertificateno("京2022112852");
        data.setZYear("2022");
        data.setZMonth("10");
        data.setZDay("26");
        j.put("tempUrl","https://bcxin-v5-prod.obs.cn-north-1.myhuaweicloud.com/uploads/2022/10-26/__MIFnVvYFZZjw5fN3OtS/1666772764122/qualicerttemp_zgz.html");
        j.put("formData",data);
        String result = HttpUtil.post("http://v5inmy.test.baibaodun.cn/kms/shared/convert/htmltopdf", j.toJSONString());
        System.out.println(result);*/
    }

    private void accept(SignatureQueuesDTO queue) {
        Long a = System.currentTimeMillis();
        SignatureData data = JSONObject.parseObject(queue.getData(),SignatureData.class);
        //先将文件命名为.temp
        data.setUnSignaturePDF(tempPDF+"/queueId_"+queue.getId()+".temp");
        String toDay = data.getFzDate();
        if(StringUtils.isEmpty(toDay)){
            toDay = DateUtil.today();
        }
        data.setZYear(toDay.split("-")[0]);
        data.setZMonth(toDay.split("-")[1]);
        data.setZDay(toDay.split("-")[2]);
        data.setHeadphoto(data.getHeadphoto().startsWith("http")?data.getHeadphoto():urlHead+data.getHeadphoto());

        String result = "";
        SharedConvertDto sharedConvertDto = new SharedConvertDto();
        sharedConvertDto.setTempUrl(queue.getTemplate_url());
        sharedConvertDto.setFormData(data);
        try {
            //最终输出为.pdf文件，主要是为了避免pdf 0kb时就被剪切走
            result = convertService.execConvertHtmltopdfForTemp(sharedConvertDto);
        } catch (IOException e) {
            result = "";
            log.error("====> 电子签章定时任务开始.createPDF.accept.pdf生成异常："+e.getMessage());
        }
        if(StringUtils.isNotEmpty(result)){
            //成功
            signatureQueuesRepository.updateSignature("3",new Date(),"PDF已生成待签章",queue.getId());
        }else{
            //失败
            signatureQueuesRepository.updateSignature("2",new Date(),"PDF生成失败",queue.getId());
        }
        log.info("====> 电子签章定时任务开始.createPDF.accept.单次生成文件耗时："+(System.currentTimeMillis()-a));
    }

    public void delInPDF(){
        SharedConvertDto sharedConvertDto = new SharedConvertDto();
        sharedConvertDto.setTempUrl(tempPDF);
        convertService.delInPDF(sharedConvertDto);
    }

    @Override
    public void signQuicklyExCGI() {
        Long a = System.currentTimeMillis();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("tempUrl", tempPDF);
        jsonObject.put("formData", 230);
        try {
            //到文件夹剪切一个文件到out目录
            SharedConvertDto sharedConvertDto = new SharedConvertDto();
            String tempUrl = jsonObject.getString("tempUrl");
            String formData = jsonObject.getString("formData");
            sharedConvertDto.setTempUrl(tempUrl);
            sharedConvertDto.setFormData(formData);
            new Thread(()->{
                try {
                    //先检索in目录存在的删除
                    convertService.delInPDF(sharedConvertDto);
                } catch(Exception e) {
                    log.error("====> 电子签章定时任务开始.signQuicklyExCGI签章前比对in文件删除："+ e.getMessage());
                }
            }).start();
            //剪切文件到in目录下
            convertService.existsPdfCopy(sharedConvertDto);
            //每次盖章前先清除0kb文件
            convertService.delInPDF(tempPDF);
            //先触发对需要签章目录下文件进行签章
            activateXMPSignQuicklyExCGI();
            new Thread(()->{
                try {
                    //先检索in目录存在的删除
                    convertService.delInPDF(sharedConvertDto);
                } catch(Exception e) {
                    log.error("====> 电子签章定时任务开始.signQuicklyExCGI签章后比对in文件删除："+ e.getMessage());
                }
            }).start();
            log.info("====> 电子签章定时任务开始.signQuicklyExCGI.签章接口耗时："+(System.currentTimeMillis()-a));
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Override
    public void changeStatus() {
        String updateSql = "update obpm2_security.signature_queues set status = '0' where status = '3' and TIMESTAMPDIFF(HOUR, last_processed_time, NOW()) > 3";
        jdbcTemplate.update(updateSql);
    }

    public void scan() {
        StringBuilder sb = new StringBuilder();
        try {
            Long a = System.currentTimeMillis();
            JSONObject jsonObject = new JSONObject();
            String pdfFile = "/uploads/pdf/dianziqianzhang/out";
            jsonObject.put("tempUrl", pdfFile);
            jsonObject.put("formData", null);
            SharedConvertDto sharedConvertDto = new SharedConvertDto();
            sharedConvertDto.setTempUrl(pdfFile);
            sharedConvertDto.setFormData(null);
            String result = convertService.existsScanSignature(sharedConvertDto);
            if (StringUtils.isEmpty(result)) {
                log.info("====> 电子签章定时任务开始.scan.result：空");
                return;
            }
            log.info("====> 电子签章定时任务开始.scan.批量上传ftp结束耗时：" + (System.currentTimeMillis() - a));
            JSONObject json = JSONObject.parseObject(result);
            Object success = json.get("success");
            Object fail = json.get("fail");
//            String successQidS = "";
//            String failQidS = "";
            List<String> failList = null;
            String successFile = "";
            String failFile = "";
            if (success != null){
                successFile = (String) success;//queueId_134147.pdf,queueId_134148.pdf,queueId_134149.pdf
                /*if(StringUtils.isNotEmpty(successFile)){
                    successQidS = "(" + successFile.replace(".pdf", "'").replace("queueId_", "'") + ")";
                }*/
            }
            if (fail != null){
                failFile = (String) fail;//queueId_134147.pdf,queueId_134148.pdf,queueId_134149.pdf
                if(StringUtils.isNotEmpty(failFile)){
//                    failQidS = "(" + failFile.replace(".pdf", "'").replace("queueId_", "'") + ")";
                    String[] failS = failFile.replace(".pdf", "").replace("queueId_", "").split(",") ;
                    if(failS.length>0){
                        failList = Arrays.asList(failS);
                    }
                }
            }
            String fileNames = successFile+failFile;
            new Thread(()->{
                try {
                    //只要ftp有跑过的都要比对删除
                    convertService.delSetFtpFile(tempPDF,fileNames);
                } catch(Exception e) {
                    log.error("====> 电子签章定时任务开始.scan.ftp操作过的文件删除："+ e.getMessage());
                }
            }).start();

            if (StringUtils.isNotEmpty(successFile)) {
                String updateSql;
                //成功
                String[] successSplit = successFile.split(",");
                for (String string : successSplit) {
                    String successFilePath = pdfFile+"/"+string;
                    String id = string.replace("queueId_","").replace(".pdf","");
                    String business_id = signatureQueuesRepository.getBuId(id);
                    updateSql = "UPDATE tlk_certificate SET ITEM_SIGNATURED_PROCESSED_STATUS=\"2\",ITEM_LAST_SIGNATURED_PROCESSED_TIME=NOW(),ITEM_CERTIFICATEDATE=IFNULL(ITEM_CERTIFICATEDATE,NOW()),ITEM_LAST_SIGNATURED_PROCESSED_RESULT=\"完成\",ITEM_ATTACHMENT=\"" + successFilePath + "\" WHERE ID=\"" + business_id + "\";";
                    jdbcTemplate.update(updateSql);
                    signatureQueuesRepository.updateSignature("1", new Date(), "成功", id);
                }
            }
            if (failList != null){
                signatureQueuesRepository.updateSignatures("2", new Date(), "未成功签章", failList);
            }
            log.info("====> 电子签章定时任务开始.scan.50个PDF已签章文件存FTP耗时：" + (System.currentTimeMillis() - a));
        } catch (Exception e) {
            log.error("====> 电子签章定时任务开始.scan.updateSql error：" + ExceptionUtils.getStackMessage(e));
        }
    }

    @Data
    public static class SignatureData {
        private String name;
        private String address;
        private String headphoto;
        private String idcardno;
        private String year;
        private String month;
        private String day;
        private String certificateno;
        private String isSignature;
        //发证日期
        private String fzDate;
        private String zYear;
        private String zMonth;
        private String zDay;
        private String signatureImg;
        //未签章pdf存储路径
        private String unSignaturePDF;
        //已签章pdf扫描路径
        private String signaturePDF;
    }

    /**
     * Restful 接口返回的资源对象
     *
     */
    @Data
    public static class Resource {
        private int errcode;
        private String errmsg;
        private Object data;
    }
}
