package com.bcxin.backend.domain.syncs.services.impls;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.backend.domain.apps.AppDefinition;
import com.bcxin.backend.domain.apps.TriggerDefinition;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;

@Service
@Slf4j
public class GetAppTaskJobServiceImpl {

    @Value("${myapps.domain.url}")
    private String URL;
    //private String URL = "http://v5qy.test.baibaodun.cn/obpm";


    @Cacheable(value = "triggerList")
    public List<TriggerDefinition> getApps() throws Exception {

        String GETAPPURL = URL + "/api/v2/runtime/definitions/apps";
        String TRIGGERSURL = URL + "/api/v2/runtime/definitions/apps/%s/triggers";
        List triggerList = new ArrayList<TriggerDefinition>();
        Set<TriggerDefinition> triggerSet = new HashSet<>();
        log.error("=====================走请求了================");
        try {
            HttpRequest httpRequest = HttpUtil.createGet(GETAPPURL);
            HttpResponse execute = httpRequest.execute();
            String appsJsonStr = execute.body();
            if (!ObjectUtils.isEmpty(appsJsonStr) && execute.isOk()) {
                List<AppDefinition> apps = JSONObject.parseArray(execute.body(), AppDefinition.class);
                if (!ObjectUtils.isEmpty(apps)) {
                    apps.stream().forEach(appDefinition -> {
                        String format = String.format(TRIGGERSURL, appDefinition.getId());
                        boolean ok = false;
                        String tggResStr = null;
                        try {
                            HttpRequest tggReq = HttpUtil.createGet(format);
                            HttpResponse tggRes = tggReq.execute();
                             tggResStr = tggRes.body();
                            ok = tggRes.isOk();
                        }catch (Exception e){
                            e.printStackTrace();
                        }
                        List<TriggerDefinition> list = new ArrayList<>();
                        if (ok) {
                            list = JSONObject.parseArray(tggResStr, TriggerDefinition.class);
                        } else {
                            log.error("获取定时任务ID出现异常:"+tggResStr);
                        }
                        if (!ObjectUtils.isEmpty(list)) {
                            list.forEach(tir -> {
                                triggerSet.add(tir);
                            });
                        }
                    });
                    triggerList.addAll(triggerSet);
                }
            }else {
                log.error(appsJsonStr);
                System.err.println(appsJsonStr);
            }
        } catch (Exception e) {
            //如果请求失败，就清除缓存
            log.error("获取连接失败，执行清除缓存");
            e.printStackTrace();
            throw e;
        }
        return triggerList;
    }


    @CacheEvict(value = "triggerList")
    public void cleanAppTaskIdsCache() {
        log.error("缓存已清除");
    }

}
