package com.bcxin.backend.domain.utils.ftp;

import com.bcxin.backend.domain.configs.FileModeConfig;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * <b> 批量上传 </b>
 * @author ZXF
 * @create 2023/07/07 0007 13:43
 * @version
 * @注意事项 </b>
 */
public class FtpUtil {

    /**
     * 上传文件
     *
     * @param ftpPath
     * @param listFiles
     * @return
     */

    public static synchronized List<UploadResult> upload(String ftpPath, File[] listFiles) {
        String cont = "";
        //构建线程池
        ExecutorService newFixedThreadPool = Executors.newFixedThreadPool(FileModeConfig.threadPoolSize);
        List<Future> results = new ArrayList<>();
        //创建n个ftp链接
        FtpFactory factory = new FtpFactory();
        for (File file : listFiles) {
            //根据文件最后修改时间，如果小于5分钟的不进行上传（可能文件还没写入流）
            long a = System.currentTimeMillis();
            long createTime = file.lastModified();
            if(((a-createTime)/60/1000)<5){
                continue;
            }
            FtpConnection ftp = factory.getFtp();//获取ftp con
            UploadTask upload = new UploadTask(factory,ftp, file, ftpPath, file.getName());
            Future submit = newFixedThreadPool.submit(upload);
            results.add(submit);
        }

        List<UploadResult> listResults = new ArrayList<>();
        for (Future result : results) {
            try {
                //获取线程结果
                UploadResult uploadResult = (UploadResult)result.get(30, TimeUnit.MINUTES);
                listResults.add(uploadResult);
            } catch (Exception e) {
                if(StringUtils.isEmpty(cont)&&e.getMessage().contains("SocketException: Broken pipe")){
                    cont = e.getMessage();
                }
                System.err.println("=======>线程池上传ftp异常："+e.getMessage()+"<=======");
//                e.printStackTrace();
            }
        }
        factory.close();
        newFixedThreadPool.shutdown();
        if(StringUtils.isNotEmpty(cont)){
            listResults = null;
        }
        return listResults;
    }
}
